// Copyright  2017 Autodesk, Inc.
// All rights reserved.

/**
 * Simple data plotter
 */

var cfgPane_autoCurPlot = true;
var logXOn = false;
var logYOn = false;
var dbOn = false;

// toggle logscale property of plot and save state
function doLogX(btn)
{
    checkLogOK();
    if (!logXCapable) {
        alert("The x-axis data cannot be plotted on a log scale");
        return;
    }

    // check all plots
    logXOn = !logXOn;
    dygraphPlot.updateOptions({axes: {x: {logscale: logXOn}}});
    btn.value = "LogX " + (!logXOn ? "ON" : "OFF");
}

// toggle logscale property of plot and save state
function doLogY(btn)
{
    var err = false;
    checkLogOK();

    if (!logYOn && !logYCapable) { // only check if it is not on
        alert("The y-axis data cannot be plotted on a log scale, use 'Select Signals' to filter" + (dbOn ? ", and try turning off dB" : ""));
        return;
    }

    // check all plots and update
    logYOn = !logYOn;
    dygraphPlot.updateOptions({logscale: logYOn});
    btn.value = "LogY " + (!logYOn ? "ON" : "OFF");
}

function makeImage() {
    var newImgComp1 = $('#plot1img')[0];
    Dygraph.Export.asPNG(dygraphPlot, newImgComp1, options);
    newImgComp1.setAttribute("width", 500);
    newImgComp1.setAttribute("height", p1i_h * 500 / p1i_w); // 620x460, 620/500=460/h\
    //window.location.href = newImgComp1.src.replace('image/png','image/octet-stream');
    $("#plotDiv").hide();
    $("#imgDiv").show();
}

function refreshPlot()
{
    document.location = document.location;
}

function log(s) {
    console.log(s);
}

// landscape (note ipad gives width as portrait width)
var theScreenWidth = parseInt(screen.height > screen.width ? screen.height : screen.width);
var theScreenHeight = parseInt(screen.height < screen.width ? screen.height : screen.width);

var simResult = "";
function postDataLoad()
{
    if (simResult == null || simResult == "") {
        return;
    }

    simResult = simResult.trim(); // new, needed because keep-alive feature on server sends spaces
    savSimRes = simResult;

    // seperate output and data
    var output = simResult.substring(simResult.indexOf("FL=") + 3, simResult.length);

    var isOP = false;

    // op analysis will go after FL
    if (output.indexOf("{NV:") != -1) {
        var ind1 = output.indexOf("{NV:");
        var ind2 = output.indexOf("}", ind1);
        if (ind1 != -1 && ind2 != -1) {
            var opRes = output.substring(ind1, ind2 + 1).trim();
            opRes = null;
            isOP = true;
        }
        ind1 = ind2 = null;
    }
    if (output.indexOf("{BC:") != -1) {
        var ind1 = output.indexOf("{BC:");
        var ind2 = output.indexOf("}", ind1);
        if (ind1 != -1 && ind2 != -1) {
            var opRes = output.substring(ind1, ind2 + 1).trim();
            opRes = null;
            isOP = true;
        }
        ind1 = ind2 = null;
    }

    // check for errors
    if (simResult.toLowerCase().indexOf("aborted") != -1 ||
        simResult.toLowerCase().indexOf("error in simulation") != -1 ||
        simResult.toLowerCase().indexOf("fatal error") != -1) {
        alert("Simulation was aborted, please check simuation results and/or netlist for errors");
    }
    else if (isOP) {
        opValuesOn = false;
        showOpValues(); // toggle
    }
    else {
        // plot, then go to sim tab
        plot();

        // turn on log x and db if ac analysis
        if(isACAnalysis) {
            doLogX($("#logXBtn")[0]);
            doDB($("#dbBtn")[0]);
        }

        if (lastNumSignals > 10) {
            alert("There are more than 10 signals plotted.  For best performance, consider  " +
                "removing some probes/ports and/or setting less signals to be plotted  " +
                "from the Plot Options menu.  ");
        }

        plottedYet = null;
    }

    isOP = null;
    output = null;
}

function finalReformat()
{
    $("#topDiv").css("height", "90%");
    $("#topDiv").css("width", "99%");
    $("#plotleg").css("padding-left", "10px");
    $("#plotleg").css("border-left-color", "grey");
    $("#plotleg").css("border-left-style", "solid");
    $("#plotleg").css("border-left-width", "2px");
    $("#plotleg").css("height", "99%");
    $("#plotOptionsDiv").css("padding-left", "10px");
    $("#plotOptionsDiv").css("border-left-color", "grey");
    $("#plotOptionsDiv").css("border-left-style", "solid");
    $("#plotOptionsDiv").css("border-left-width", "2px");
    $("#plotOptionsDiv").css("height", "99%");
}

$(document).ready(function() {

    simResult = "FL="; // sim result from spice

    try {
        setTimeout(function(){
            $.getJSON("simdata/simdata.json", function (data) {
                $.each(data, function (key, val) {
                    simResult = key + "=" + val + "\n" + simResult;
                });

                $("#plotWrapper").css("display", "none");
                $("#plotButtons").css("display", "none");
                $("#loadingImg").css("display", "block");
                postDataLoad();
                finalReformat();
                $("#loadingImg").css("display", "none");
                $("#plotButtons").css("display", "block");
                $("#plotWrapper").css("display", "block");
            });
        }, 1000); // give it a second will ya?
    } catch(e) {
        simResult = "";
    }
});

function hideSidebar(id)
{
    $('#plot1Cover').hide();
    $('#plotleg').show();
    $('#' + id).hide();
    $('#plotButtons').show();
}

function clearHistory()
{
    for(var location = 1; location <= 5; location++) {
        localStorage.removeItem("SIM" + location);
        localStorage.removeItem("SIMNAME" + location);
    }
    refreshPlot();
}

function clearSelected()
{
    var location = $("#RECALLLIST").val();
    if (location === "") {
        return;
    }
    location = parseInt(location, 10);

    if (location === 0) {
        for(location = 1; location <= 5; location++) {
            localStorage.removeItem("SIM" + location);
            localStorage.removeItem("SIMNAME" + location);
        }
    } else {
        localStorage.removeItem("SIM" + location);
        localStorage.removeItem("SIMNAME" + location);
    }
    setHistoryOptions();
}


function saveTo()
{
    var location = parseInt($("#SAVETOLIST").val(), 10);
    var saveName = $("#saveName").val();
    saveName = saveName.replace("=", ":");
    saveName = saveName.replace(" ", "_");

    // prefix all signals with memory location name
    var savSimResTmp = savSimRes.slice(0); // copy
    savSimResTmp = savSimResTmp.split("\n");
    for(var i = 0; i < savSimResTmp.length; i++) {
        if(savSimResTmp[i].indexOf("FL=") == -1) {
            var parts = savSimResTmp[i].split("=");
            var nm = parts[0];
            if(nm.indexOf("x-axis") === -1) {
                savSimResTmp[i] = "{" + (saveName != "" ? saveName : location) + "}" + nm + "=" + parts[1]; // reassemble
            }
        }
    }

    var simResToStore = "";
    for(var i = 0; i < savSimResTmp.length; i++) {
        simResToStore += savSimResTmp[i] + "\n";
    }

    localStorage.setItem("SIM" + location, simResToStore);
    localStorage.setItem("SIMNAME" + location, saveName.trim());
    setHistoryOptions();
}

function recallFrom()
{
    var location = $("#RECALLLIST").val();
    if (location === "") {
        return;
    }
    location = parseInt(location, 10);

    if(location === 0) {
        // build a huge sim result
        var bigSimResult = "";
        for(var location = 1; location <= 5; location++) {
            var curSimResult = localStorage.getItem("SIM" + location);
            if(curSimResult != null && curSimResult != "") {
                if (curSimResult.indexOf("FL=") != -1) {
                    curSimResult = curSimResult.substring(0, curSimResult.indexOf("FL="));
                }

                if(location != 1) {
                    // find x-axis and delete it
                    var ind1 = curSimResult.indexOf("x-axis");
                    var ind2 = curSimResult.indexOf("\n", ind1);
                    curSimResult = curSimResult.substring(0, ind1) +
                        curSimResult.substring(ind2, curSimResult.length);
                }
                bigSimResult += curSimResult + "\n";
            }
        }

        simResult = bigSimResult.replace("\n\n", "\n");
        simResult += "\nFL=";

    } else {
        simResult = localStorage.getItem("SIM" + location);
    }
    postDataLoad();
    finalReformat();
}

function setHistoryOptions() {
    // init
    $("#saveName").val("");
    var select = $("#SAVETOLIST")[0];
    select.options[0].selected = true;
    select = $("#RECALLLIST")[0];
    select.options[1].selected = true;

    // find names
    for (var i = 1; i < select.length; i++){ // skip first one
        var option = select.options[i];
        // now have option.text, option.value
        var nm = localStorage.getItem("SIMNAME" + i);
        if(nm != null && nm != "") {
            option.text = "Set " + i + " - " + nm;
        } else {
            option.text = "Set " + i;
        }
    }
}


/***************************************************************
 * Function: plot
 * Params: n/a
 * DESC: plotting function, uses dygraph library
 * http://dygraphs.com/tests/two-axes.html
 ***************************************************************/
var xdata = [];
var xname = "";
var thedata = [];
var viData = [];
var dygraphPlot = null;
var labelNames = [];
var labelsToRemove = [];  // all labels
var labelsToRemove2 = []; // array of true false values for each label
var logXOn = false;
var logYOn = false;
var logXCapable = true;
var logYCapable = true;
var options = null;
var isACAnalysis = false;
var savSimRes = ""; // last sim result
function plot() {
    // clear axes fields
    logXOn = false;
    $("#logXBtn").val("LogX ON");
    logYOn = false;
    $("#logYBtn").val("LogY ON");

    isACAnalysis = false;
    {
        $('#plotWrapper').show();

        if (dygraphPlot != null) {
            dygraphPlot.destroy();
            dygraphPlot = null;
        }

        labelsToRemove = [];  // all labels
        labelsToRemove2 = []; // array of true false values for each label

        xdata = [];
        xname = "";
        thedata = [];

        var res = savSimRes;

        if (res.indexOf("x-axis:") != -1) {
            res = res.substring(0, res.indexOf("FL="));
            res = res.split("\n")

            try {
                for (i = 0; i < res.length; i++) {
                    if (res[i] != "") {
                        var ind = res[i].indexOf("=");
                        var nm = res[i].substring(0, ind);
                        nm = nm.trim(); // very important when using " " on server for wait time

                        // format history name
                        var historyName = "";
                        if (nm.indexOf("{") == 0) {
                            var ind2 = nm.indexOf("}");
                            historyName = nm.substring(0, ind2+1);
                            historyName = historyName.substring(1, historyName.length-1);
                            nm = nm.substring(ind2+1, nm.length); // remove
                        }

                        // remove current identifier
                        if (nm.indexOf("#br") != -1) {
                            nm = nm.substring(0, nm.indexOf("#br"));
                            if (nm.indexOf("I(") != 0) {
                                nm = "I(" + nm + ")";
                            }
                        }

                        var xaxis = 0
                        if (nm.indexOf("x-axis:") == 0) {
                            xname = nm.substring(nm.indexOf("x-axis:") + 7, nm.length);
                            xaxis = 1;

                            if(xname == "frequency") {
                                isACAnalysis = true;
                            }

                            xname = (xname == "time" ?
                                "Time (s)" :
                                (xname == "frequency" ?
                                    "Frequency (Hz)" :
                                    xname));
                        }

                        // show history name
                        nm = (historyName != "" ? "[" + historyName + "] " : "") + nm;

                        var dta = res[i].substring(ind + 1, res[i].length);
                        dta = dta.split(",");

                        //log("data: " + nm + " " + dta.length);

                        dataset = [];
                        for (j = 0; j < dta.length; j++) {
                            dta[j] = dta[j].trim();
                            if (dta[j] != null && dta[j] != "") {
                                if (xaxis === 1) {
                                    xdata.push(parseFloat(dta[j]));
                                    //log("xdata " + dta[j] + " " + parseFloat(dta[j]));
                                }
                                else {
                                    if (dbOn && nm.indexOf("vp(") == -1 && nm.indexOf("I(") == -1) {
                                        try {
                                            dataset.push(20.0 * Math.log10(parseFloat(dta[j])));
                                        } catch(logexp) {
                                            dataset.push(1e-12); // dummy so plot still works
                                        }
                                    } else {
                                        dataset.push(parseFloat(dta[j]));
                                    }
                                }
                            }
                        }

                        if (xaxis === 0) {
                            thedata.push([nm, dataset]);
                        }
                    }
                }

                log("xdata " + xdata.length);
                log("thedata " + thedata.length);

                // PLOT
                viData = [];

                labelNames = [xname];

                //var notPhaseNamesV = [];

                // to fix the phase discontinuity
                var lastPhase = {};
                var phaseMod = {};

                // which datapoint
                for (i = 0; i < xdata.length; i++) {
                    x = xdata[i];
                    viData.push([x]);

                    // which set
                    for (j = 0; j < thedata.length; j++) {
                        y = ((thedata[j])[1])[i];
                        ylabel = (thedata[j])[0];

                        if (ylabel.indexOf("I(") != -1) {
                            //iData[i].push(y);
                            viData[i].push(y);
                            if (i == 0) {
                                //labelNamesI.push(ylabel);
                                labelNames.push(ylabel);
                                labelsToRemove.push(ylabel);

                                if (cfgPane_autoCurPlot) {
                                    labelsToRemove2.push(true);
                                }
                                else {
                                    labelsToRemove2.push(false);
                                }
                            }
                        }
                        else {
                            {
                                if (ylabel.indexOf("vp(") == 0) {
                                    if (i == 0) // initialize
                                    {
                                        phaseMod[j] = 0.0;
                                        lastPhase[j] = y * 180.0 / 3.14159;
                                    }

                                    y = phaseMod[j] + y * 180.0 / 3.14159; // convert to degrees

                                    // going over discontinuity
                                    if ((lastPhase[j] < -135 && y > 135)) {
                                        phaseMod[j] -= 360.0;
                                        y += phaseMod[j];
                                    }
                                    else if ((lastPhase[j] > 135 && y < -135)) {
                                        phaseMod[j] += 360.0;
                                        y += phaseMod[j];
                                    }
                                    lastPhase[j] = y;
                                }

                                viData[i].push(y);
                            }

                            if (i == 0) {
                                labelNames.push(ylabel);
                                labelsToRemove.push(ylabel);
                                labelsToRemove2.push(true);
                            }
                        }
                    }
                }

                lastPhase = null;
                phaseMod = null;

                // for making secondary axis
                var seriesObj = {};

                // visibility array
                var vArr1 = [];
                for (va = 0; va < labelNames.length; va++) {
                    vArr1.push(true);
                    if (va > 0 && (labelNames[va].indexOf("I(") != -1 ||
                            labelNames[va].indexOf("vp(") == 0)) {
                        seriesObj[labelNames[va]] = {
                            axis: 'y2'
                        };
                    }
                    else if (va > 0 &&
                        (labelNames[va].indexOf("v(") == 0 ||
                            labelNames[va].indexOf("vdb(") == 0)) {
                        seriesObj[labelNames[va]] = {
                            axis: 'y'
                        };
                    }
                }

                var colorArr = colorScheme(labelNames.length);

                // combined options
                options = {
                    labels: labelNames,
                    height: "90%",
                    width: "90%",
                    title: " ", // gives a little space on top
                    titleHeight: 24,
                    xlabel: xname,
                    ylabel: (dbOn ? 'Voltage (dBV)' : 'Voltage (V)'),
                    y2label: 'Current (A)' + (isACAnalysis ? ' / Phase Angle (Degrees)' : ''),
                    series: seriesObj,
                    drawPoints: true,
                    pointSize: 1,
                    highlightCircleSize: 4,
                    labelsDiv: 'plotleg',
                    labelsSeparateLines: true,
                    showLabelsOnHighlight: true,
                    stackedGraph: false,
                    colorSaturation: 0.8,
                    visibility: vArr1,
                    hideOverlayOnMouseOut: false,
                    legend: "always",
                    sigFigs: 4,
                    //digitsAfterDecimal: 6,
                    strokeWidth: 2,
                    axisLabelWidth: 100,
                    drawAxesAtZero: true,
                    colors: colorArr,
                    legendFormatter: legendFormatter,
                    zoomCallback: function(minDate, maxDate, yRange) {
                        myZoomCallback(minDate, maxDate, yRange);
                    }
                };
                colorArr = null;

                // save # signals and limit them in plot
                lastNumSignals = thedata.length;
                seriesObj = null;

                var plotW = $("#plot1")[0].width;
                var plotH = $("#plot1")[0].height;
                $("#plot1").css("width", plotW+"px");
                $("#plot1").css("height", plotH+"px");

                dygraphPlot = new Dygraph(document.getElementById("plot1"), viData, options);
                doPlotUpdateAdv();
                advancedPlotting();

                // format for csv
                var csvStr = "";
                for (var xInd = 0; xInd < labelNames.length; xInd++) {
                    if (xInd > 0) {
                        csvStr += ",";
                    }
                    csvStr += labelNames[xInd].replace(/,/g, "/");
                }
                csvStr += "\n";
                for (var xInd = 0; xInd < viData.length; xInd++) {
                    for (var xInd2 = 0; xInd2 < viData[xInd].length; xInd2++) {
                        if (xInd2 > 0) {
                            csvStr += ",";
                        }
                        csvStr += viData[xInd][xInd2];
                    }
                    csvStr += "\n";
                }
                csvStr = null;

                //notPhaseNamesV = null;

                checkLogOK();

                $("canvas").on("contextmenu",function(){
                    return false;
                });

            }
            catch (e) {
                alert("There was a problem getting simulation results");
                log(e.message);
            }
        }
        else {
            //alert("No data for plot, check simulation results for error messages");
        }
    }
}

/***************************************************************
 * Function: checkLogOK
 * Params: n/a
 * DESC: check if plotting in log scale is ok
 ***************************************************************/
function checkLogOK() {
    // make sure to do this after the db is applied
    logXCapable = true;
    logYCapable = true;

    for (var xInd = 0; xInd < viData.length; xInd++) {
        for (var xInd2 = 0; xInd2 < viData[xInd].length; xInd2++) {
            if(xInd2 === 0 && viData[xInd][xInd2] < 0.0) { // x-axis is not in labelsToRemove2 so don't check for that here
                logXCapable = false;
            }
            // need labelsToRemove2[xInd2-1] since labelsToRemove2 does not include x-axis
            else if((xInd2 > 0) && (labelsToRemove2[xInd2-1] === true) && (viData[xInd][xInd2] <= 0.0)) {
                logYCapable = false;
            }
        }
    }
}


/***************************************************************
 * Function: doPlotUpdateAdv
 * Params: linesOn, ptsOn
 * DESC: custom plotting options helper functions
 ***************************************************************/
function doPlotUpdateAdv() {
    // plot 1 (skip x axis name)
    for (i = 1; i < labelNames.length; i++) {
        for (j = 0; j < labelsToRemove.length; j++) {
            if (labelsToRemove[j] === labelNames[i]) {
                // set series i to invisible
                dygraphPlot.setVisibility((i - 1), labelsToRemove2[j]);
            }
        }
    }

    checkLogOK();
    resizePlots();

    // reverse it, then click button so we can go through checks
    logYOn = !logYOn;
    doLogY($("#logYBtn")[0]);
};

function labelHelper(ind, checkbox) {
    labelsToRemove2[ind] = checkbox.checked;
}

// helper fnuction
function currentsOff() {
    // turn off currents
    for (cInd = 0; cInd < labelsToRemove.length; cInd++) {
        if (labelsToRemove[cInd].indexOf("I(") == 0) {
            labelsToRemove2[cInd] = false;
        }
    }
}

// return number of signals on, stop when > 10
function countNumSignalsPlotted(limitOn) {
    var cnt = 0;
    for (cInd = 0; cInd < labelsToRemove2.length; cInd++) {
        if (labelsToRemove2[cInd] === true) {
            cnt += 1;

            if (cnt > 10 && limitOn) {
                labelsToRemove2[cInd] = false; // hide this signal
            }
        }
    }
    return cnt;
}

// limit the number of signals plotted to 10
function limitNumSignalsPlotted() {
    countNumSignalsPlotted(true);
}

// make sure plot is appropriately sized
var lastNumSignals = 0;
var p1i_h = 0;
var p1i_w = 0;
function resizePlots() {
    $("#plotWrapper").css("width", "99%");
    p1i_h = ($('#plotWrapper').height() - 80);
    p1i_w = (($('#plotWrapper').width() * 7 / 10));
    return true;
}

/**
 * Reset plot zoom to automatic values and update fields
 */
function resetPlotZoom() {
    dygraphPlot.updateOptions({ dateWindow: null });
    dygraphPlot.updateOptions({ axes: { y: {valueRange: null } }});
    dygraphPlot.updateOptions({ axes: {y2: {valueRange: null } }});
    prepopRanges();
}


/**
 * This function gets a value, trims it, attempts to parse to float
 * @param obj
 * @returns {*}
 */
function setRangeHellper(obj) {
    var val = obj.val();
    try {
        val = val.trim();
        val = (val != "" ? parseFloat(val) : "");
        if (val != "" && isNaN(val)) {
            err = true;
        }
    } catch(e) {
        // set to empty field where there is error, graph will put default value
        obj.val("");
    }

    return val;
}

/**
 * update DOM fields on zoom change
 * @param minDate
 * @param maxDate
 * @param yRanges
 */
function myZoomCallback(minDate, maxDate, yRanges) {
    $('#XMIN').val(minDate);
    $('#XMAX').val(maxDate);
    $('#Y1MIN').val(yRanges[0][0]);
    $('#Y1MAX').val(yRanges[0][1]);
    $('#Y2MIN').val(yRanges[1][0]);
    $('#Y2MAX').val(yRanges[1][1]);
}

/**
 * Return true if the value is NOT an empty string and IS a number
 * @function checkEmptyOrNan
 * @param val
 * @returns {boolean}
 */
function checkValidNumber(val)
{
    return !(val === "") && !isNaN(val);
}

/**
 * Function: setRange
 * Params: n/a
 * DESC: set range for plot
 */
function setPlotRange() {
    var xmin = setRangeHellper($('#XMIN'));
    var xmax = setRangeHellper($('#XMAX'));
    var y1min = setRangeHellper($('#Y1MIN'));
    var y1max = setRangeHellper($('#Y1MAX'));
    var y2min = setRangeHellper($('#Y2MIN'));
    var y2max = setRangeHellper($('#Y2MAX'));

    if (checkValidNumber(xmin) && checkValidNumber(xmax)) {
        dygraphPlot.updateOptions({ dateWindow: [xmin, xmax]} );
    }
    if (checkValidNumber(y1min) && checkValidNumber(y1max)) {
        dygraphPlot.updateOptions({ axes: { y: {valueRange: [y1min, y1max]} } } );
    }
    if (checkValidNumber(y2min) && checkValidNumber(y2max)) {
        dygraphPlot.updateOptions({axes: {y2: {valueRange: [y2min, y2max]}}});
    }

    return true;
}

function setRangeDiv() {
    var newHt = '<b>Set Plot Range</b><br>' +
        '<table cellpadding="1" cellspacing="1" width="100%" id="plotRangeTable">';
    // plot range settings

    newHt += '<tr><td align="left">X Min</td><td align="right"><input id="XMIN" type="text" class="simBtn" value="" maxlength="25" size="10"></td></tr>';
    newHt += '<tr><td align="left">X Max</td><td align="right"><input id="XMAX" type="text" class="simBtn" value="" maxlength="25" size="10"></td></tr>';
    newHt += '<tr><td align="left">Y Min</td><td align="right"><input id="Y1MIN" type="text" class="simBtn" value="" maxlength="25" size="10"></td></tr>';
    newHt += '<tr><td align="left">Y Max</td><td align="right"><input id="Y1MAX" type="text" class="simBtn" value="" maxlength="25" size="10"></td></tr>';
    newHt += '<tr><td align="left">Y2 Min</td><td align="right"><input id="Y2MIN" type="text" class="simBtn" value="" maxlength="25" size="10"></td></tr>';
    newHt += '<tr><td align="left">Y2 Max</td><td align="right"><input id="Y2MAX" type="text" class="simBtn" value="" maxlength="25" size="10"></td></tr>';
    newHt += '<tr><td align="center" colspan="2">';
    newHt += '<input class="simBtn" type="button" id="plotRangeRstBtn" value="Reset" onclick="javascript: { resetPlotZoom(); }">&nbsp;';
    newHt += '<input class="simBtn" type="button" id="plotRangeBtn" value="Apply Changes" onclick="javascript: { if (setPlotRange()) { $(\'#plot1Cover\').hide(); $(\'#plotleg\').show(); $(\'#plotRangeDiv\').hide(); $(\'#plotButtons\').show(); } }"><br>';
    newHt += '</td></tr>';
    newHt += '<tr><td align="left" colspan="2">';
    newHt += '<p id="simPlotRangeMsg">Enter values in decimal or scientific notation format (e.g., 0.001 or 1e-3)</p>';
    newHt += '</td></tr>';
    newHt += '</table>';
    if ($('#plotRangeDiv').html() == "") {
        $('#plotRangeDiv').html(newHt);
    }
    prepopRanges();
}

/***************************************************************
 * Function: advancedPlotting
 * Params: n/a
 * DESC: custom plotting options html generation and plot range settings fields
 * note this must be called after plot() even if not visible at first so the DOM exists
 ***************************************************************/
function advancedPlotting() {
    if (thedata.length > 0) {
        var newHt = '<b>Select signals to plot</b><br><br>' +
            '<table cellpadding="2" cellspacing="2" width="100%" id="plotSelectTable">';
        for (i = 0; i < labelsToRemove.length; i++) {
            newHt += '<tr style="background-color: #EEE">' +
                '<td align="center" valign="bottom" width="5%">' +
                '    <input type="checkbox" id="onOffCheckboxes' + (i) + 'a" name="onOffCheckboxes' + (i) + 'a" ' + (labelsToRemove2[i] ? 'checked="checked"' : '') + ' onchange=\'javascript: labelHelper(' + i + ', this); \' />' +
                '</td>' +
                '<td align="left" valign="top">' + labelsToRemove[i] + '</td>';
            newHt += '</tr>';
        }
        newHt += '<tr><td colspan="2" align="center"><br>' +
            '<input class="simBtn" type="button" value="Select All" onclick="javascript: { setAllCheckboxes(true); } "/>&nbsp;' +
            '<input class="simBtn" type="button" value="Clear All" onclick="javascript: { setAllCheckboxes(false); } "/>&nbsp;' +
            '<input class="simBtn" type="button" value="Apply Changes" onclick="javascript: { $(\'#plot1Cover\').hide(); doPlotUpdateAdv(); advancedPlotting(); $(\'#plotleg\').show(); $(\'#plotOptionsDiv\').hide(); $(\'#plotButtons\').show(); }"/>' +
            '</td></tr>'
        newHt += '</table><br>';
        $('#plotOptionsDiv').html(newHt);
    }
    else {
        alert('There is no data to modify');
    }
}

function setAllCheckboxes(val)
{
    var ind = 0;
    $('input:checkbox').each(function(){
        $(this).prop('checked',val);
        labelHelper(ind++, this);
    });
}

function doDB(btn)
{
    // save log states
    var saveLogXOn = logXOn;
    var saveLogYOn = logYOn;

    var saveLabelsToRemove = labelsToRemove.slice();
    var saveLabelsToRemove2 = labelsToRemove2.slice();

    if(dbOn) {
        dbOn = false;
        btn.value = "dB ON";
    } else {
        dbOn = true;
        btn.value = "dB OFF";
    }

    plot();

    // put back to how it was
    if(saveLogXOn != logXOn) {
        $("#logXBtn").click();
    }
    if(saveLogYOn != logYOn) {
        $("#logYBtn").click();
    }

    labelsToRemove = saveLabelsToRemove.slice();
    labelsToRemove2 = saveLabelsToRemove2.slice();
    doPlotUpdateAdv();
    advancedPlotting();
    setRangeDiv();
}

/**
 * Set range fields based on plot data ranges
 * @function prepopRanges
 */
function prepopRanges() {
    $('#XMIN').val(dygraphPlot.xAxisRange()[0]);
    $('#XMAX').val(dygraphPlot.xAxisRange()[1]);
    $('#Y1MIN').val(dygraphPlot.yAxisRange(0)[0]);
    $('#Y1MAX').val(dygraphPlot.yAxisRange(0)[1]);
    $('#Y2MIN').val(dygraphPlot.yAxisRange(1)[0]);
    $('#Y2MAX').val(dygraphPlot.yAxisRange(1)[1]);
}

/**
 * HSV to RGB color conversion
 *
 * H runs from 0 to 360 degrees
 * S and V run from 0 to 100
 *
 * Ported from the excellent java algorithm by Eugene Vishnevsky at:
 * http://www.cs.rit.edu/~ncs/color/t_convert.html
 */
function hsvToRgb(h, s, v) {
    var r, g, b;
    var i;
    var f, p, q, t;

    // Make sure our arguments stay in-range
    h = Math.max(0, Math.min(360, h));
    s = Math.max(0, Math.min(100, s));
    v = Math.max(0, Math.min(100, v));

    // We accept saturation and value arguments from 0 to 100 because that's
    // how Photoshop represents those values. Internally, however, the
    // saturation and value are calculated from a range of 0 to 1. We make
    // That conversion here.
    s /= 100;
    v /= 100;

    if (s == 0) {
        // Achromatic (grey)
        r = g = b = v;
        return [Math.round(r * 255), Math.round(g * 255), Math.round(b * 255)];
    }

    h /= 60; // sector 0 to 5
    i = Math.floor(h);
    f = h - i; // factorial part of h
    p = v * (1 - s);
    q = v * (1 - s * f);
    t = v * (1 - s * (1 - f));

    switch (i) {
        case 0:
            r = v;
            g = t;
            b = p;
            break;

        case 1:
            r = q;
            g = v;
            b = p;
            break;

        case 2:
            r = p;
            g = v;
            b = t;
            break;

        case 3:
            r = p;
            g = q;
            b = v;
            break;

        case 4:
            r = t;
            g = p;
            b = v;
            break;

        default: // case 5:
            r = v;
            g = p;
            b = q;
    }

    return [Math.round(r * 255), Math.round(g * 255), Math.round(b * 255)];
}

// helper to make colors for plot
function colorScheme(total) {
    var i = 360 / (total - 1); // distribute the colors evenly on the hue range
    var r = []; // hold the generated colors
    for (var x = 0; x < total; x++) {
        var rgb = hsvToRgb(i * x, 75, 75);
        r.push("rgb(" + rgb[0] + "," + rgb[1] + "," + rgb[2] + ")"); // you can also alternate the saturation and value for even more contrast between the colors
        rgb = null;
    }
    return r;
}

function legendFormatter(data) {
    if (data.x == null) {
        // This happens when there's no selection and {legend: 'always'} is set.
        return '<br>' + data.series.map(function(series) { return series.dashHTML + ' ' + series.labelHTML }).join('<br>');
    }

    var html = this.getLabels()[0] + ': ' + data.xHTML;
    data.series.forEach(function(series) {
        if (!series.isVisible) return;
        var labeledData = series.labelHTML + ': ' + series.yHTML;
        if (series.isHighlighted) {
            labeledData = '<b>' + labeledData + '</b>';
        }
        html += '<br>' + series.dashHTML + ' ' + labeledData;
    });
    return html;
}
